package cn.com.jdkj.aspect;

import cn.com.jdkj.enums.ReturnCode;
import cn.com.jdkj.exception.CustomException;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateUtil;
import cn.hutool.crypto.digest.DigestUtil;
import com.alibaba.fastjson.JSONObject;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.format.annotation.DateTimeFormat;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestAttributes;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.lang.reflect.Field;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;

@Aspect
@Component
@Slf4j
//@RequiredArgsConstructor
public class CheckSignAspect {

//    String appid="1717726426";

    String secret="089f7f8a1d6186524c2b4cdb0b4fb5e1";

    @Pointcut("@within(org.springframework.web.bind.annotation.RestController)&&@annotation(cn.com.jdkj.annotation.CheckSign))")
    public void controlerPointCut() {
    }

    @Around("controlerPointCut()")
    public Object accessTokenToUserInfo(ProceedingJoinPoint pjp) throws Throwable {
        //上下文获取请求头 获取token
        RequestAttributes requestAttributes = RequestContextHolder.getRequestAttributes();
        HttpServletRequest request = ((ServletRequestAttributes)requestAttributes).getRequest();
        String sign = request.getHeader("sign");
        String appid = request.getHeader("appid");
        String t = request.getHeader("t");
        //获取传参数据
        Object[] args = pjp.getArgs();
        //获取传参名
//        MethodSignature msg = (MethodSignature) pjp.getSignature();
        SortedMap<String, String> paramsMap = new TreeMap<>();
        for (Object arg : args) {
            if (arg instanceof HttpServletRequest) {
                continue;
            }
            if (arg instanceof HttpServletResponse) {
                continue;
            }
            Class<?> clazz = arg.getClass();
            Field[] fields = clazz.getDeclaredFields();
            for (Field field : fields) {
                field.setAccessible(true);
                Object o = field.get(arg);
                if (Objects.nonNull(o)) {
                    String value = formatValue(field, o);
                    paramsMap.put(field.getName(), value);
                }
            }
        }
        paramsMap.put("appid",appid);
        paramsMap.put("t",t);
        String toSign = mapToSign(paramsMap)+secret;
        log.info("toSing:{}",toSign);
        String checkSign = DigestUtil.md5Hex(toSign);
        boolean result = checkSign.equalsIgnoreCase(sign);
        log.info("计算签名结果:{},传参签名:{},是否一致:{}",checkSign,sign,result);
        if (!result){
            throw new CustomException(ReturnCode.SIGN_ERROR);
        }
        return pjp.proceed(args);
    }

    String mapToSign(SortedMap<String,String> map){
        StringBuffer sb = new StringBuffer();
        map.entrySet().forEach(item->{
            sb.append("&");
            sb.append(item.getKey());
            sb.append("=");
            sb.append(item.getValue());
        });
        return sb.substring(1);
    }

    private String formatValue(Field field, Object o) {
        String value;
        if (o instanceof Date) {
            DateTimeFormat annotation = field.getAnnotation(DateTimeFormat.class);
            String pattern;
            if (Objects.nonNull(annotation)) {
                pattern = annotation.pattern();
            } else {
                pattern = DatePattern.NORM_DATETIME_PATTERN;
            }
            value = DateUtil.format((Date) o, pattern);
        } else if (o instanceof LocalDateTime) {
            value = DateUtil.format((LocalDateTime) o, DatePattern.NORM_DATETIME_PATTERN);
        } else {
            value = String.valueOf(o);
        }
        return value;
    }
}
