package cn.com.jdkj.config;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.context.ApplicationContextAware;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.UUID;

/**
 *
 * @ClassName: FrontInterceptor
 * @Description: 全局前置拦截器
 * @Author jyy
 * @DateTime 2022年3月29日 上午9:35:14
 */
public class FrontInterceptor extends HandlerInterceptorAdapter implements ApplicationContextAware {

	private static final Logger log = LoggerFactory.getLogger(FrontInterceptor.class);

	private static final String UNIQUE_ID = "traceId";
	// 必须声明为static
	private static ApplicationContext context;

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler)
			throws Exception {

    	String traceID = request.getHeader("TRACE_ID");
		if(StringUtils.isEmpty(traceID))
			traceID = UUID.randomUUID().toString().replace("-", "");
		MDC.put(UNIQUE_ID, traceID);

		//log.info(request.getRequestURI() + " ->> request ip：" + request.getRemoteHost() + ": " + request.getRemotePort()
		//+ " BIRD-MAC-ADDRESS:" + request.getHeader("BIRD-MAC-ADDRESS"));

		return true;

	}

	@Override
	public void setApplicationContext(ApplicationContext applicationContext) throws BeansException {
		if (context == null) {
			context = applicationContext;
		}
	}

	@Override
	public void postHandle(HttpServletRequest httpServletRequest, HttpServletResponse httpServletResponse, Object o, ModelAndView modelAndView) throws Exception {
		MDC.remove(UNIQUE_ID);
	}

}
