package cn.com.jdkj.config;

import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisPassword;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceClientConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.connection.lettuce.LettucePoolingClientConfiguration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.listener.RedisMessageListenerContainer;
import org.springframework.data.redis.serializer.Jackson2JsonRedisSerializer;
import org.springframework.data.redis.serializer.RedisSerializer;
import org.springframework.data.redis.serializer.StringRedisSerializer;
import redis.clients.jedis.JedisPool;
import redis.clients.jedis.JedisPoolConfig;

import java.time.Duration;


/**
 * Created by liujinbang on 2017/4/17.
 */
@Configuration
public class RedisConfiguration {
	@Value("${spring.redis.host}")
	private  String host;
	@Value("${spring.redis.port}")
	private int port;
	@Value("${spring.redis.timeout}")
	private int timeout;
	@Value("${spring.redis.password}")
	private  String password;
	@Value("${spring.redis.database}")
	private   int database;

	@Bean
	@Autowired
	public   JedisPool jedisPool(@Qualifier("redis.clients.jedis") JedisPoolConfig config) {
		return new JedisPool(config, host, port, timeout, password,database);
		//return new JedisPool(config, host, port);
	}


	@Bean(name = "redis.clients.jedis")
	public JedisPoolConfig jedisPoolConfig(@Value("${spring.redis.jedis.pool.max-active}") int maxTotal,
										   @Value("${spring.redis.jedis.pool.max-idle}") int maxIdle,
										   @Value("${spring.redis.jedis.pool.max-wait}") int maxWaitMillis) {
//		Config config = new Config();
//		config.setCodec(new JsonJacksonCodec());
		JedisPoolConfig config = new JedisPoolConfig();
		config.setMaxTotal(maxTotal);
		config.setMaxIdle(maxIdle);
		config.setMaxWaitMillis(maxWaitMillis);
		return config;
	}


	@Bean
	RedisMessageListenerContainer container(RedisConnectionFactory connectionFactory) {

		RedisMessageListenerContainer container = new RedisMessageListenerContainer();
		container.setConnectionFactory(connectionFactory);
		return container;
	}
//
	@Bean
	public RedisTemplate<String, Object> redisTemplate(LettuceConnectionFactory connectionFactory) {
		//  配置 Jackson2JsonRedisSerializer 序列化器，在配置 redisTemplate需要用来做k,v的序列化器
		//此种序列化方式结果清晰、容易阅读、存储字节少、速度快，所以推荐更换
		Jackson2JsonRedisSerializer<Object> jackson2JsonRedisSerializer = new Jackson2JsonRedisSerializer<Object>(Object.class);
		RedisSerializer<?> stringSerializer = new StringRedisSerializer();


		RedisTemplate<String, Object> redisTemplate = new RedisTemplate<>();
		//GenericJackson2JsonRedisSerializer jackson2JsonRedisSerializer = new GenericJackson2JsonRedisSerializer();
		// 设置键（key）的序列化采用StringRedisSerializer。
		redisTemplate.setKeySerializer(stringSerializer);
		// 设置值（value）的序列化采用jackson的序列化。
		redisTemplate.setValueSerializer(jackson2JsonRedisSerializer);
		redisTemplate.setHashKeySerializer(stringSerializer);
		redisTemplate.setHashValueSerializer(jackson2JsonRedisSerializer);
		redisTemplate.setConnectionFactory(connectionFactory);
		return redisTemplate;
	}
	@Bean
	public LettuceConnectionFactory lettuceConnectionFactory() {
		GenericObjectPoolConfig genericObjectPoolConfig = new GenericObjectPoolConfig();
		genericObjectPoolConfig.setMaxIdle(5);
		genericObjectPoolConfig.setMinIdle(5);
		genericObjectPoolConfig.setMaxTotal(8);
		genericObjectPoolConfig.setMaxWaitMillis(1000);
		genericObjectPoolConfig.setTimeBetweenEvictionRunsMillis(100);
		RedisStandaloneConfiguration redisStandaloneConfiguration = new RedisStandaloneConfiguration();
		redisStandaloneConfiguration.setDatabase(database);
		redisStandaloneConfiguration.setHostName(host);
		redisStandaloneConfiguration.setPort(port);
		redisStandaloneConfiguration.setPassword(RedisPassword.of(password));
		LettuceClientConfiguration clientConfig = LettucePoolingClientConfiguration.builder()
				.commandTimeout(Duration.ofMillis(timeout))
				.poolConfig(genericObjectPoolConfig)
				.build();

		LettuceConnectionFactory factory = new LettuceConnectionFactory(redisStandaloneConfiguration, clientConfig);
	//  factory.setShareNativeConnection(true);
	//  factory.setValidateConnection(false);
		return factory;
	}
//private final Decoder<Object> decoder = new Decoder<Object>() {
//	@Override
//	public Object decode(ByteBuf buf, State state) throws IOException {
//		ByteBufInputStream in = new ByteBufInputStream(buf);
//
//		FSTObjectInput inputSteam = config.getObjectInfut(in);
//
//	}
//}
}

