package cn.com.jdkj.customer.service.imp;

import cn.com.jdkj.config.RedissonConfig;
import cn.com.jdkj.constant.TokenKey;
import cn.com.jdkj.customer.dto.*;
import cn.com.jdkj.customer.entity.Customer;
import cn.com.jdkj.customer.mapper.CustomerMapper;
import cn.com.jdkj.customer.service.BaiduService;
import cn.com.jdkj.customer.service.CustomerService;
import cn.com.jdkj.customer.vo.AIResponseBodyVo;
import cn.com.jdkj.entity.ResultBean;
import cn.com.jdkj.enums.AIMessageTypeEnums;
import cn.com.jdkj.enums.ReturnCode;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.http.HttpResponse;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.redisson.api.RBucket;
import org.redisson.api.RedissonClient;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;
import org.springframework.util.StreamUtils;

import javax.servlet.http.HttpServletResponse;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
@Slf4j
@RequiredArgsConstructor
public class CustomerServiceImpl extends ServiceImpl<CustomerMapper, Customer> implements CustomerService {

    final RedissonClient redisson;
    final BaiduService baiduService;
    @Override
    public ResultBean login(LoginDto dto) {
        Customer customer = getOne(Wrappers.<Customer>lambdaQuery().eq(Customer::getUsername,dto.getUsername()).eq(Customer::getDelFlag,0));
        if (customer==null){
            return ResultBean.error(ReturnCode.USER_NOT_EXSIT);
        }
        if (!Integer.valueOf(1).equals(customer.getActiveFlag())){
            return ResultBean.error(ReturnCode.USER_NOT_ACTIVE);
        }
        if (!customer.getPassword().equals(dto.getPassword())){
            return ResultBean.error(ReturnCode.PASSWORD_ERROR);
        }
        String uuid = null;
        do{
            uuid = UUID.randomUUID().toString().replace("-","");
        }while (redisson.getBucket(TokenKey.USER_KEY_ID+uuid).get()!=null);
        redisson.getBucket(TokenKey.USER_KEY_ID+uuid).set(customer.getId(),8, TimeUnit.HOURS);
        return ResultBean.succsess(uuid);
    }

    @Override
    public ResultBean register(RegisterDto dto) {
        Long count = count(Wrappers.<Customer>lambdaQuery().eq(Customer::getUsername,dto.getUsername()).eq(Customer::getDelFlag,0));
        if (count>0){
            return ResultBean.error(ReturnCode.USERNAME_EXSIT);
        }
        // TODO: 2024/6/6 用户名违规
        Customer customer = new Customer();
        BeanUtils.copyProperties(dto,customer);
        customer.setActiveFlag(1);
        customer.setDelFlag(0);
        customer.insert();
        LoginDto loginDto = new LoginDto();
        loginDto.setUsername(dto.getUsername());
        loginDto.setPassword(dto.getPassword());
        return login(loginDto);
    }
    boolean checkUUID(String uuid){
        RBucket rBucket = redisson.getBucket(TokenKey.USER_KEY_ID+uuid);
        if (rBucket.get()==null){
            return false;
        }
        return true;
    }
    @Override
    public ResultBean question(QuestionDto dto) {
        //验证uuid
        if (!checkUUID(dto.getUUID())){
            return ResultBean.error(ReturnCode.UUID_ERROR);
        }
        return ResultBean.succsess(question(dto.getQs()));
    }

    String question(String text){
        HttpRequest request = HttpRequest.post("https://book.9jodia.net/v1/chat/completions");
        request.header("Authorization","Bearer sk-w6IHvBqZBvrHfSFg23217c734c7247C990C5F528D66a6691");
        AIBodyDto body = new AIBodyDto();
        AIMessageDto sys = new AIMessageDto();
        sys.setContent("你是一个工作助手，请精简回答问题，回答字数不超过100字");
        sys.setRole(AIMessageTypeEnums.system.getCode());
        AIMessageDto messageDto = new AIMessageDto();
        messageDto.setContent(text);
        messageDto.setRole(AIMessageTypeEnums.user.getCode());
        List<AIMessageDto> messages = ListUtil.of(sys,messageDto);
        body.setMessages(messages);
        request.body(JSONObject.toJSONString(body));
        Long start = System.currentTimeMillis();
        HttpResponse response = request.execute();
        Long end = System.currentTimeMillis();
        log.info("问题长度:{},耗时:{}ms",text.length(),end-start);
//        log.info("response:{}",response);
        AIResponseBodyVo responseBodyVo = JSONObject.parseObject(response.body()).toJavaObject(AIResponseBodyVo.class);
        StringBuffer sb = new StringBuffer();
        responseBodyVo.getChoices().forEach(item->{
            sb.append(item.getMessage().getContent());
        });
        return sb.toString();
    }

    @Override
    public void questionToAudio(QuestionDto dto, HttpServletResponse response) {
        //验证uuid
        if (!checkUUID(dto.getUUID())){
            ResultBean.error(ReturnCode.UUID_ERROR);
        }
        InputStream inputStream = baiduService.text2audio(question(dto.getQs()));
        try {
            byte[] data = new byte[1024];
            response.setContentType("application/octet-stream");
            response.setHeader("Content-Disposition","attachment; filename=\""+System.currentTimeMillis()+".mp3\"");
            response.setHeader("accept-Ranges","bytes");
            int read = 0;
            while ((read=inputStream.read(data))!=-1){
                response.getOutputStream().write(data,0,read);
            }
            response.getOutputStream().flush();
        }catch (Exception e){
            log.error("copy stream error:{}",e.getMessage(),e);
        }
    }

}
