package cn.com.jdkj.util;

import org.apache.commons.codec.digest.DigestUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.regex.Pattern;

/**
 * 字符串相关工具类
 *
 * @author HuangChendi
 */
public abstract class StringUtils extends org.springframework.util.StringUtils {

    public static String generateCode() {
        String result = "";
        Random random = new Random();
        for (int i = 0; i < 20; i++) {
            result += random.nextInt(10);
        }
        return result;
    }

    public static String generateCode(int length) {
        String result = "";
        Random random = new Random();
        for (int i = 0; i < length; i++) {
            result += random.nextInt(10);
        }
        return result;
    }

    /**
     * 处理逗号分隔的字符串
     *
     * @param str
     * @return
     */
    public static String dealString(String str) {
        String res = "";
        if (!isEmpty(str)) {
            String[] ids = str.split(",");
            for (String id : ids) {
                if (!isEmpty(id))
                    res = res.length() == 0 ? id : res + "," + id;
            }
        }
        return res;
    }

    /**
     * 处理逗号分隔的字符串
     *
     * @param str
     * @return
     */
    public static List<Integer> splitString(String str) {
        List<Integer> res = new ArrayList<>();
        if (!isEmpty(str)) {
            String[] ids = str.split(",");
            for (String id : ids) {
                id = id.replaceAll(" ", "");
                if (!StringUtils.isEmpty(id)) {
                    res.add(Integer.valueOf(id));
                }
            }
        }
        return res;
    }

    /**
     * 处理逗号分隔的字符串
     *
     * @param str
     * @return
     */
    public static String dealString(List<?> ids) {
        String res = "";
        for (Object id : ids) {
            if (!StringUtils.isEmpty(id))
                res = res.length() == 0 ? id + "" : res + "," + id;
        }
        return res;
    }

    public static String dealString(Object[] ids) {
        String res = "";
        for (Object id : ids) {
            if (!StringUtils.isEmpty(id))
                res = res.length() == 0 ? id + "" : res + "," + id;
        }
        return res;
    }

    /**
     * 判断是否为整数
     *
     * @param str 传入的字符串
     * @return 是整数返回true, 否则返回false
     */
    public static boolean isInteger(String str) {
        Pattern pattern = Pattern.compile("^[\\d]*$");
        return pattern.matcher(str).matches();
    }

    /**
     * @param @param  number
     * @param @return 设定文件
     * @return Object    返回类型
     * @throws
     * @Description: 判断数字是否是整数
     */
    public static Boolean getNumber(Float number) {
        if (number.intValue() - number.floatValue() == 0) {
            return true;
        } else {
            return false;
        }
    }


    /**
     * 判断传入参数,如果是Null或者空值，返回false，不为空返回true
     *
     * @param checkAll True:所有的为空才返回,False:只要有一个为空返回
     * @param strings
     * @return
     */
    public static Boolean checkNull(Boolean checkAll, Object... objects) {
        Boolean ret = true;
        if (objects == null) return false;
        for (Object s : objects) {
            if (null == s || ("").equals(s.toString().trim())) {
                if (!checkAll)
                    return false;
                else {
                    ret = false;
                }
            }
        }
        return ret;
    }

    public static int getMax(String[] arr) {
        int max = Integer.valueOf(arr[0]);
        for (int x = 1; x < arr.length; x++) {
            if (Integer.valueOf(arr[x]) > max)
                max = Integer.valueOf(arr[x]);
        }
        return max;
    }

    public static int getMin(String[] arr) {
        int min = Integer.valueOf(arr[0]);
        for (int x = 1; x < arr.length; x++) {
            if (Integer.valueOf(arr[x]) < min)
                min = Integer.valueOf(arr[x]);
        }
        return min;
    }

    /**
     * 生成规则序号
     *
     * @param equipmentType 设备类型
     * @param equipmentNo   最新设备编号
     * @return
     */
    public static String getNewEquipmentNo(String equipmentNo) {
        String newEquipmentNo = "001";
        if (equipmentNo != null && !equipmentNo.isEmpty()) {
            int newEquipment = Integer.parseInt(equipmentNo) + 1;
            newEquipmentNo = String.format("%03d", newEquipment);
        }
        return newEquipmentNo;
    }

    /**
     * 截取字符串的最后几位
     *
     * @param len 截取的长度
     * @param str 被截取的字符串
     * @return
     */
    public static String getSubString(int len, String str) {
        String substring = str.substring(str.length() - len, str.length());
        return substring;
    }

    /**
     * @param @param  v1
     * @param @param  v2
     * @param @return 参数说明
     * @return int    返回类型
     * @throws
     * @Title: compareVersion
     * @Description: TODO(字符串版本号比较大小)
     */
    public static int compareVersion(String v1, String v2) {
        if (v1.equals(v2)) {
            return 0;
        }
        String[] version1Array = v1.split("[._]");
        String[] version2Array = v2.split("[._]");
        int index = 0;
        int minLen = Math.min(version1Array.length, version2Array.length);
        long diff = 0;

        while (index < minLen
                && (diff = Long.parseLong(version1Array[index])
                - Long.parseLong(version2Array[index])) == 0) {
            index++;
        }
        if (diff == 0) {
            for (int i = index; i < version1Array.length; i++) {
                if (Long.parseLong(version1Array[i]) > 0) {
                    return 1;
                }
            }

            for (int i = index; i < version2Array.length; i++) {
                if (Long.parseLong(version2Array[i]) > 0) {
                    return -1;
                }
            }
            return 0;
        } else {
            return diff > 0 ? 1 : -1;
        }

    }



    // 拼接字符串
    public static String concatenatedString(String... str) {
        StringBuilder sb = new StringBuilder();
        for (String s : str) {
            if (null != s && !s.isEmpty()) {
                sb.append(s);
            }
        }
        return sb.toString();
    }

    public static boolean isNotEmpty(String macAddr) {
        return !isEmpty(macAddr);
    }

    public static String buildOrderNumber() {
        String orderNum = new SimpleDateFormat("yyyyMMdd").format(new Date()) + StringUtils.generateCode();
        return orderNum;
    }

    public static String getOrderNum() {
        String orderNum = new SimpleDateFormat("yyyyMMdd").format(new Date()) + StringUtils.generateCode();
        return orderNum;
    }


    public static String encryptToMD5(String str) {
        return DigestUtils.md5Hex(str);
    }

    //静态变量存储最大值
    private static final AtomicInteger ATOMIC_NUM = new AtomicInteger(0);

    /**
     * @return int
     * 注：此方法并没有使用synchronized进行同步，因为共享的编号自增操作是原子操作，线程安全的
     * @Author javaloveiphone
     * @Description :获取最新分组编号
     */
    public static String getNewAutoNum() {
        //线程安全的原子操作，所以此方法无需同步
        int newNum = ATOMIC_NUM.incrementAndGet();
        //数字长度为5位，长度不够数字前面补0
        return String.format("%05d", newNum);
    }
}
